// DISK.C = Disk maintenance & utilities:
#include	"type.h"
#include	<stdio.h>	   //PROGRAMS ARE:-
errnum chkdisks(void);     			//check,make data files & datfile
FILE  *makefile(void);				//create empty data file, with header only.
errnum chkfile(FILE *ptr); 	    	//check file size,consistncy; trim if nec.
errnum trim(FILE *ptr,long offset,int num);//Trim at offset leaving num ionogs.
yesno  headok(struct ihead *ip);	//check details in ionogram header.
errnum findfree(yesno flag);		//find 1st non-full disk, dir.
errnum getfinfo(yesno flag);		//check file, print dir.
#include    "globals.h"
#include	"errors.h"
#include	"file.h"
int	dyear;

// Check existence of data files, datfile.  Make them if non-existent using
// makefile().  Repeat calls to chkfile() until consistent files result.
errnum chkdisks(void)
{	FILE *ptr;
	int e, err;
	char c;
	err = OK;
	if ( ( ptr = fopen( datfile, "r+b" ) ) == NULL )
		if ( (ptr=makefile()) == NULL ) return(EDISK);
	while ( (e=chkfile(ptr)) != 0 )
	  {	if ( e == EFULL ) break;
		if ( e == ETRIM ) continue;
		fclose( ptr );
		if ( (ptr= makefile()) == NULL) {err= EDISK; break;}  }
	if ( ptr != NULL ) fclose( ptr );
	return( err );
}//end chkdisks

// Create new empty data file with only file header in it.
FILE *makefile(void)
{	extern int gettimes(byte *time);
	extern errnum hwrite(struct ihead *hp, FILE *ptr, long off);
    extern int dsite;
	FILE *ptr;
	struct fhead fp;
	if ( (ptr= fopen(datfile,"w+b")) == NULL ) 	return(NULL);
	fp.fnum = 0;
	fp.fsize = LHSIZE;
	fp.fsite = dsite;						//0,1=Akld 2=Chch ....
	gettimes( (byte *)&fp.fyear );
	fp.fflags = 0x02;
	hwrite( (struct ihead *)&fp, ptr, 0L );
	return( ptr );
}//end *makefile

// Check file capacity and consistency, trim file if inconsistent, without
// any attempt to recover or patch up file.
errnum chkfile( ptr )
FILE *ptr;
{ 	extern errnum hwrite(struct ihead *hp, FILE *ptr, long off);
	int i, ntot;
	long off, prevoff;
	struct fhead fp;
	struct ihead ip;
	if ( fseek( ptr, 0L, 0 ) == -1 )  return(EDISK);
	if ( fread(&fp, HSIZE, 1, ptr) != 1)  return(EBADDAT);
//	if ( fp.fflags&FULL ) return( EFULL ); 	// Could check full file, but
	ntot = fp.fnum;			     			// it may get trimmed...
	prevoff = -1L;
	off = LHSIZE;
	for ( i = 0; i < ntot; i++ )
	{	if ( fseek( ptr, off, 0 ) == -1 ) return( trim(ptr,prevoff,i-1) );
		if ( fread( &ip, HSIZE, 1, ptr ) != 1 ) return( trim(ptr,off,i) );
		if ( ip.inum != i+1 )  return( trim(ptr, off, i) );
		if ( !headok( &ip ) )
			if ( hwrite( &ip, ptr, off ) )  return( trim(ptr,off,i) );
		off += ( ( long ) ( ip.isize << BLKSHFT ) ) + LHSIZE;
	}
	if ( off == fp.fsize )  return( OK );
	return( trim(ptr, off, ntot) );
}//end chkfile

// Trim file ptr at offset leaving num ionograms in file.  Note that there
// is no OK return - normal return is ETRIM, i.e. trimming successful.
// Assumes that other returns will cause makefile() to be called shortly.
errnum trim( ptr, offset, num )
FILE *ptr;
long offset;
int num;
{	extern errnum hwrite(struct ihead *hp, FILE *ptr, long off);
	struct fhead fp;
	if ( num <= 0 )  return( ETRIM );
	if ( fseek(ptr, 0L, 0) == -1 )  return( EDISK );
	if ( fread(&fp, HSIZE, 1, ptr) != 1 )  return( EDISK );
	fp.fsize = offset;
	fp.fnum = num;
	if ( hwrite((struct ihead *)&fp, ptr, 0L) )  return( EDISK );
	return( ETRIM );
}//end trim

// Check details in ionogram header.  Return TRUE if none are in error,
// or amend erroneous bits (set =0) and return FALSE.
yesno headok( ip )
struct ihead *ip;
{	yesno flag;
	flag = TRUE;
	if (ip->ilcol <0 || ip->ilcol>577)  { ip->ilcol= 0;    flag= FALSE; }
	if (ip->ircol <0 || ip->ircol>577)  { ip->ircol=577;   flag= FALSE; }
	if (ip->iheight<0 ||ip->iheight>64) { ip->iheight= 64; flag= FALSE; }
	if (ip->imonth > 12)  { ip->imonth = 0;  flag = FALSE; } //bytes, >= 0.
	if (ip->iday   > 31)  { ip->iday   = 0;  flag = FALSE; }
	if (ip->ihour  > 23)  { ip->ihour  = 0;  flag = FALSE; }
	if (ip->iminute> 59)  { ip->iminute= 0;  flag = FALSE; }
	if (ip->isecond> 59)  { ip->isecond= 0;  flag = FALSE; }
	return( flag );
}//end headok

// Find disk with CURRENT bit in fflags set, or 1st non-full disk.
// Search all drives as for chkdisks(), but without the laborious checking.
// Do a dir of contents of each file if flag != 0 (in findfree() only).
// Error returns:  OK if prev CURRENT file is found; ENEW if new CURRENT
// file is found; EFULL if file system is full;  others= various disk errors.
errnum findfree( flag )
yesno flag;
{	extern errnum hwrite(struct ihead *hp, FILE *ptr, long off);
	FILE *ptr;
	int e;
	char c, fcf, fff;			// First current file, first free file resp.
	struct fhead fp;			// Should only be one of the former.
	fcf = fff = 0;	c = 1;
	e = getfinfo( flag );
	if (e == ECURRENT)  { fcf= c; fff= c; }
	if (e == OK || e == ENEW)   { fff= c; }
	if (e ==ENODAT && (ptr=makefile()) != NULL)  fclose(ptr);  ////	}
	if ( fcf )  return( OK );
	if ( fff )
	  {	if ( (ptr= fopen( datfile, "r+b")) == NULL)  return( EDISK );
		if ( fread( &fp, HSIZE, 1, ptr ) != 1 )
			{ fclose( ptr );  return( EBADDAT );  }
		if ( hwrite( (struct ihead *)&fp, ptr, 0L ) != OK )
			{ fclose( ptr );  return( ENOWRITE ); }
		fclose( ptr );
		return( ENEW );  }
	return( e );
}//end findfree

// If flag is set, print dir of ionogs on disk.  Show errs in file state.
errnum getfinfo( flag )
yesno flag;
{	FILE *ptr;
	int i, e, ntotl;
	long off;
	struct fhead fp;
	struct ihead ip;
	if ( (ptr= fopen(datfile,"rb")) == 0 )  return( ENODAT );
	if ( fread(&fp,HSIZE,1,ptr)!= 1)  { fclose(ptr); return(EBADDAT); }
	dyear = fp.fyear;
	if ( !flag )
	  {	fclose( ptr );
		if (fp.fflags&CURRENT) return(ECURRENT);
		if ( fp.fnum == 0 )    return(ENEW);
		if ( fp.fflags&FULL )  return(EFULL);
		return( OK ); }
	if ( (ntotl= fp.fnum) == 0 )
	  {	fclose(ptr);
		if ( fp.fflags&CURRENT )  {
			printf("\nFile %s is empty but in use.\n", datfile);
			return( ECURRENT );   }
		printf("\nFile %s is empty.",  datfile);
		return(ENEW);  }
	else printf("\nIonograms in file:  %s  are:\n", datfile);
					//-------------- List ionograms in current file
	printf("No.        DATE           TIME          FREQUENCIES  HEIGHTS   BYTES");
	off = LHSIZE;                                 //size of headers = 16 bytes
	for ( i = 1; i <= ntotl; i++ )
	{	if ( fseek(ptr,off,0) == -1 )    { fclose(ptr); return(EBADDAT); }
		if (fread(&ip,HSIZE,1,ptr)!= 1)  { fclose(ptr); return(EBADDAT); }
		if  (ntotl>10 && i>4 && i<ntotl-3) { if (i==5) printf("\n,,,,,"); }
		else  {  printf( "\n%d\t", ip.inum );
			printf( "%02d/%02d/%4d\t",  ip.iday,  ip.imonth,  dyear );
			printf( "%02d:%02d:%02d\t", ip.ihour, ip.iminute, ip.isecond);
			printf("%d\t%d\t%d\t%d", ip.ilcol,ip.ircol, ip.iheight,ip.isize*16 ); }
		off += ( (long) (ip.isize << BLKSHFT) ) + LHSIZE;       //-> next header
	}
	fclose(ptr);  e = OK;
	if (fp.fflags & CURRENT)
		{ printf("\nThis is the currently used data file."); e = ECURRENT; }
	if (fp.fflags & FULL)  { printf("\nFile is full."); e = EFULL; }
	return( e );
}//end getfinfo  ret findfree


