//----------------------------------------------------------------------------
//            UNIVERSITY  OF  AUCKLAND   'DIGION'  SYSTEM:
//		     'GETION'  IPS-42 IONOSONDE  CONTROL  SOFTWARE
//	     (1)  Paul Qualtrough,  Radio Physics M.Sc., 1987.
//	     (2)  Nigel Lewin,  Computer Technician,  1992-93.
//	     (3)  John Titheridge,  1993--.
//            Physics Department,  University of Auckland.
///////////////// COPYRIGHT 1993  by  J. E. TITHERIDGE ///////////////////////
//----------------------------------------------------------------------------
// Uses files .c: getion, control, scan, signal, savcom, disk, display, time.
//            .h: file, globals, iono, signals, type, errors.
//           and  sounds.asm  using  prologue.h.
//----------------------------------------------------------------------------
// Ionogram size is set in VIDSCAN.
// iodesp->flag: bit1= 1/0 for 1st/later freqs (extends 1st wait in sounds).
//----------------------------------------------------------------------------
// USE: time_interval -ve to get fewer messages during operation.
// time_interval = 90 + n  uses dummy data, at each nth freq.
//               odd,even values -> data pattern F00F, E32C(111...11..1.11..).
//               -ve reads hardware data (from dipswitch), every nth freq.
// Uses: Port Addresses  240 to 243 hex,  set in  SCAN.C  and  SOUNDS.ASM.
//----------------------------------------------------------------------------
// GETION.C:  Functions in this file:
void main(int argc, char * argv[]);	//->RESET; INITIALISE; CONTROL(in control)
void make_filename(void);   //make ionog file "datfile"= mmmyy.dd eg FEB92.12
void reset(void);			//hardware reset of log file,interface,time,disk.
void prompt_time(void);		//get & show date,time.  ask if OK, exit if not.
int answer_me(int i, int character); //returns chr from kbd; or i after 1 min.
#include	"type.h"
errnum initialise(void);    //initialise software.  disk error->beep?, retry.
#include	"file.h"
#include	"errors.h"
#include	"signals.h"
#include	<stdio.h>
#include    <conio.h>
#include    <stdlib.h>
#include    <string.h>
#include	<dos.h>

int     dsite= 1; //-->> CHANGE to set site number for all ionograms in DISK
				   //  OR read from start-up file  digion.bat
byte    ernum;			//Used to find error exit point in SOUND()
unsigned rdynum;		// Used to give data readys received in SOUND()
char    datfile[30];	// Storage for the ionogram filename
struct 	iodes iode,*iodesp = &iode;
int		day, mon, yr;	// used as indicator for new file creation after a
        				// new day; external in globals, & used in CONTROL().
int 	time_interval;  // The time between programmed soundings.
int 	time_offset;    // Start at ( hour + time_offset ) mins.
int     debug = 1;		// zero only at time_interval -ve, to reduce lists
int		noclean= 0;		//-ve clean all; 0/1/n leave: hrly/ all/ every nth.

void main(int argc, char * argv[])
{   int i, trun = 0;
	extern void control(void);          //3'94: add initial port initialisatn.
 	  outportb( CTRLPORT, INITBYTE );   //initialise 8255; A,B,Chi=in, Clo=out
	  outportb( CTRLPORT, SCANOFBYTE);  //set line R = ScanRequest to Off
	  i = atoi(getenv("SITENUM"));      //Is site# set as environment var?
	  if (i>0) dsite= i;
	clrscr();
	printf("\n   DIGION system,  used with an IPS-42 Ionosonde.");
	printf("\n   Version 4.  Copyright 1993 by J.E. Titheridge,");
	printf("\n   Physics Dept.,  University of Auckland,   N.Z.");
	printf("\n      Ionosonde is addressed at PortA =%4x(hex).\n", PORTA);
	printf("\n   GETION.EXE of March 1994,  for Station No. %2d.\n", dsite);
	time_interval = time_offset = 0;
	if (argc>1)  time_interval= atoi( argv[1] );
	if (argc>2)  time_offset  = atoi( argv[2] );
	if (argc>2)  noclean = atoi( argv[3] );
	if (time_interval!=0)  printf("\nRun with time interval %d min,  "
						  "offset %d min.\n", time_interval, time_offset);
				//--------- SET TIME, SCAN INTERVAL, FILENAME, and then
	if (argc==2 && time_interval==0)  { strcpy(datfile, argv[1]);
		printf("\nONE IONOGRAM will be stored in FILE: %s. ",datfile);
		trun= 3;  time_interval= -1;  }
	else prompt_time();                                 //(-> def OK in 1min)
	if (argc<=1)
	  {	printf("\nEnter the desired time interval between soundings ? ");
		printf("\n [1, 2, 3, 5,...,60 minutes.   Default = 5 mins] ");
		time_interval = answer_me(5, 0);  }     		//0->int; def= 5 mins
	debug = time_interval;
	if (time_interval<0)  { time_interval= -time_interval;  debug= 0; }
	if (debug)  printf(" Run with full listings.  ");
	if (trun==0) make_filename();
//----------------------------- RUN, repeatedly:  Reset, Initialise are below.
	for (;;)  {	reset();	   //=below: open logfile, init 8255, send scanReq.
		initialise();		   //=below: locate & open ionogram file.
		control();		       //control: Wait for time & call getscan, display.
		if (trun==3) exit(trun); }    //exit with 1 ionog stored in given file.
}//end main

// Construct the ionogram file, named "datfile" = mmmyy.dd, eg FEB92.12
void make_filename(void)
{	char temp[5];    ///[3]
	const char *month[] = {"JAN", "FEB", "MAR", "APR", "MAY", "JUN",
						   "JUL", "AUG", "SEP", "OCT", "NOV", "DEC" };
	struct date today;
	getdate (&today);
	mon = (int)today.da_mon;
	day = (int)today.da_day;
	yr  = today.da_year;        			//19yy
	strcpy( datfile, month[mon-1] );
	   sprintf(temp, "%02d.%02d", yr-1900, day);
	today.da_year -= 1900;        			//yy
	strcat ( datfile, temp);
	printf("Will use data file %s", datfile);
}//end make_filename

void reset(void)	//"Hardware" reset: error log file, interface, time, disk.
{	extern errnum sync(void);
	extern errnum chkdisks(void);
	extern int   gettimes(byte *time);
	extern void  panic(char *s, int status);
	extern void  logit(char *s);
	extern yesno wait(int secdelay);
	int e, yr;
	byte c, time[7];
	char s[80];
	static int numresets;                		  //(starts zero each new day)
	printf( "\n\nEntering RESET: ");     //panic (below)-> log & exit(ABORT=0)
	if (numresets++ >10)  panic( "Resets continue, shutting down.", ABORT );
	if ( freopen( "LOGFILE.LOG", "a", stderr ) == NULL )
	  {	printf( "Error file can't be opened/created.\n" );
		fprintf(stderr,"Error file can't be opened/created.\n" ); }

	outportb( CTRLPORT, INITBYTE );     //initialise 8255; A,B,Chi=in, Clo=out
	outportb( CTRLPORT, SCANOFBYTE);    //set line R = ScanRequest to Off
	printf(" Ports are initialised.");
	yr = gettimes(time) - 1900;
	printf( "\nComputer Date= %2d-%02d-%2d",  time[3], time[2], yr );
	printf( ";  Time= %02d:%02d:%02d.",time[4], time[5], time[6] );
	printf( " \nWaiting to sync with ionosonde:  " );
	if (sync() != OK) panic( "Can't synchronise to ionosonde.", ABORT );
	system( "settime.com" );           // set the timer chip to the ionosonde time  */
	if (chkdisks()!=OK) panic("Can't read/write ionogram data file!", ABORT);
	logit ("System is reset." );
	printf(" - Leaving reset.");
}//end reset

////////// Initialise "software".  A disk error in findfree()-> beep??, retry.
		// A 2nd error -> return -> reset(),  & chkdisks() will abort (above).
#define DIR  TRUE
errnum initialise(void)
{	extern void logit(char *s);
	extern errnum empty(struct iodes *iodesp);
	extern errnum findfree(yesno flag);
	extern void clrkbd(void);
	int i, e, yr;
	byte time[7];
	printf("\nEntering disk INITIALISE() routine.");
	if ( (e= findfree(DIR)) == EFULL) {
		printf("\nFile system is full; reset,recheck."); return(1); }
	if ( e != OK && e != ENEW ) {
		logit("Some soft disk error, RESETTING!");  return(1); }
	yr = gettimes(time) - 1900;
	printf("\nTimes from INITIALISE(), as set in SYNC():");
	printf( "\n  Date= %2d-%02d-%2d.", time[3], time[2], yr );
	printf("  Time= %02d:%02d:%02d." , time[4], time[5], time[6] );
	empty(iodesp);
	clrkbd();	logit( "  Leaving INITIALISE()." );
	return( OK );
}//end initialise

void prompt_time(void)
{	int c;    ///added 5 lines to show datime - but normally done in .bat?
	  byte time[7];
	  extern int  gettimes(byte *time);
	  yr = gettimes( time ) - 1900;
	  printf("\nComputer Date= %2d-%02d-%2d;",  time[3], time[2], yr);
	  printf("  Time= %02d:%02d:%02d.",  time[4], time[5], time[6]);
	printf("\nAre these within +/-30s of the ionosonde time [Y or N]?");
	c = answer_me(89,1);  printf("\n");     				  //def chr89= 'Y'
	if (c == 'Y' || c == 'y')  return;
	else { printf("\n\nPlease use DOS to correct the time and date.\n");
		   exit(0); }
}//end promp_time

//Read decimal# (character=0), or single character, read from kybd.
int answer_me(int def, int character)     //(used getion 138, 151 only)
{                // Waits 20 SEC for response, or -> default value i.
	struct time t;
	int c, secend;
	gettime( &t );
	secend = (t.ti_sec + 20) %60;				///for 20 sec wait
	while ( !kbhit() )
	  {	gettime( &t );  if (t.ti_sec==secend)  return(def); }
	if   (!character)  scanf("%d", &c);
	else  c = getch();
	return(c);
}//end answer_me
